Domain

Struct Domain 

pub struct Domain<const COLLECTION_THRESHOLD: u8 = DEFAULT_COLLECTION_THRESHOLD> { /* private fields */ }
A hazard pointer domain that manages hazard slots and deferred reclamation.

All pointers protected and retired through the same domain share a single hazard list. Retired pointers are stored in a lock-free Treiber stack and reclaimed when no hazard slot references them.

Typically one global domain is sufficient:

use lockout_hazard::Domain;
static DOMAIN: Domain = Domain::new();

Implementations

Source

impl Domain

pub const fn new() -> Self

Creates a new hazard pointer domain.

This is a const fn, so it can be used in static declarations.

Source

impl<const COLLECTION_THRESHOLD: u8> Domain<COLLECTION_THRESHOLD>

pub const fn with_threshold() -> Self

Creates a new hazard pointer domain with a threshold for automatic pointer reclamation specified as a generic parameter.

This is a const fn, so it can be used in static declarations.

pub fn protect<T>(&self, ptr: &AtomicPtr<T>) -> Option<Guard<'_, T>>

Protects the pointer stored in a AtomicPtr by publishing it in a hazard slot.

Uses a load-reserve-verify loop to ensure the returned guard protects the value that was in ptr at the time of the call. Returns None if the pointer is null.

pub unsafe fn protect_ptr<T>(&self, ptr: *mut T) -> Option<Guard<'_, T>>

Protects an arbitrary raw pointer by publishing it in a hazard slot.

Unlike protect, this does not verify the pointer against an AtomicPtr source. The caller must ensure the pointer is valid. Returns None if the pointer is null.

Safety

The pointer must point to a valid, live allocation.

pub unsafe fn retire_ptr<T>(&self, ptr: *mut T)

Retires a raw pointer, scheduling it for deferred reclamation.

The pointer will be deallocated (via Box::from_raw) once no hazard slot references it.

Safety
  • The pointer must have been allocated with Box.
  • The pointer must no longer be reachable from any shared atomic.
  • The pointer must not be retired more than once.
pub fn collect(&self)

Scans all hazard slots and reclaims any retired pointers that are not currently protected.

This is called automatically every [COLLECTION_THRESHOLD] retires, but can also be called manually to force reclamation. Resets the retire counter.